==> gene2pathway_2.2.0.tar.gz: gene2pathway/R/run.crossvalidation.R <==
# TODO: Add comment
# 
# Author: frohlich
###############################################################################
evaluate.cv = function(alldat, cvpred, groups, signaltrans.only=FALSE, nfolds=10, DIR="."){
	labels = alldat$labels
	n = nrow(labels)	
	stats.raw = sapply(1:length(groups), function(grno){
				g = groups[[grno]]
				r = (grno-1) %/% nfolds
				myidx = r*n + g
				tp = sapply(1:ncol(labels), function(l) sum((labels[g,l] == 1) & (cvpred[myidx,l] == 1))*1)
				fp = sapply(1:ncol(labels), function(l) sum((labels[g,l] == 0) & (cvpred[myidx,l] == 1))*1)
				fn = sapply(1:ncol(labels), function(l) sum((labels[g,l] == 1) & (cvpred[myidx,l] == 0))*1)
				prec = tp / (tp + fp)
				prec[is.na(prec)] = 1
				rec = tp / (tp + fn)
				rec[is.na(rec)] = 1
				F1 = 2*prec*rec / (prec + rec)
				F1[is.na(F1)] = 0			
				
				lo = sapply(1:length(g), function(i) loss(labels[g[i],], cvpred[myidx[i],], alldat$treesizes, alldat$parentPaths))
				tp2 = sum(tp)
				fp2 = sum(fp)
				fn2 = sum(fn)
				prec.sample = tp2 / (tp2 + fp2)
				prec.sample[is.na(prec.sample)] = 1
				rec.sample = tp2 / (tp2 + fn2)
				rec.sample[is.na(rec.sample)] = 1
				F1.sample = 2*prec.sample*rec.sample / (prec.sample + rec.sample)
				F1.sample[is.na(F1.sample)] = 0		
				lss = median(lo)
				names(prec) = colnames(labels)
				names(rec) = colnames(labels)
				names(F1) = colnames(labels)						
				list(preci=prec, reca=rec, F1=F1, loss=lss, F1.sample=F1.sample, prec.sample=prec.sample, rec.sample=rec.sample)
			})				
	stats.raw = apply(stats.raw,2,unlist)
	repeats = ncol(stats.raw) / nfolds
	stats = unlist(sapply(seq(1, ncol(stats.raw), by=nfolds), function(s1) rowMeans(stats.raw[,s1:(s1+(nfolds-1))])))			
	means = rowMeans(stats)
	sds = apply(stats,1,sd,na.rm=TRUE)
		
	pdf(file=file.path(DIR, "PerformanceSummary.pdf"))
	boxplot(1-stats["loss",],stats[grep("prec\\.sample",rownames(stats)),], stats[grep("rec\\.sample",rownames(stats)),], stats[grep("F1\\.sample",rownames(stats)),], ylim=c(0,1), names=c("accuracy","precision","recall","F1"), main=paste("pathways (n = ", ncol(labels), ")"))
	dev.off()
	
	rootpaths = colnames(labels)[alldat$kegg_hierarchy$parentsLev1]
	idx = rownames(stats)[unlist(sapply(rootpaths, function(r) grep(r, rownames(stats))))]
	if(signaltrans.only)
		idx = setdiff(idx, rownames(stats)[unlist(sapply(rootpaths, function(r) grep(paste(r,"\\.[0-9]*",sep=""), rownames(stats))))])
	pdf(file=paste("PerformanceSummaryallTopHier.pdf",sep=""))
	boxplot(as.vector(stats[idx[grep("preci",idx)],]), as.vector(stats[idx[grep("reca",idx)],]), as.vector(stats[idx[grep("F1",idx)],]), ylim=c(0,1), names=c("precision","recall","F1"), main=paste("all pathways (top hierarchy, n = ", length(rootpaths), ")"))
	dev.off()		
#	perf = 	as.vector((stats[idx[grep("F1",idx)],]))
#	perf = cbind(perf, as.factor(rep(rootpaths, each=10)))
#	colnames(perf) = c("perf", "path")
#	pdf(file=file.path(DIR, "F1TopHier.pdf"))
#	boxplot(perf~path,data=perf,names=gsub("pathway", "", rootpaths),ylim=c(0,1),ylab="F1",main=paste("all pathways (top hier., n = ", length(rootpaths), ")"), las=3, cex.axis=0.5)
#	dev.off()
	
	sndpaths = colnames(labels)[alldat$kegg_hierarchy$parentsLev2]
	idx = setdiff(rownames(stats)[sapply(sndpaths, function(r) grep(r, rownames(stats)))], idx)
	pdf(file=file.path(DIR, "PerformanceSummaryall2ndHier.pdf"))
	boxplot(as.vector(stats[idx[grep("preci",idx)],]), as.vector(stats[idx[grep("reca",idx)],]), as.vector(stats[idx[grep("F1",idx)],]), ylim=c(0,1), names=c("precision","recall","F1"), main=paste("pathways (2nd hier., n = ", length(sndpaths), ")"))
	dev.off()
	
#	perf = 	as.vector((stats[idx[grep("F1",idx)],]))
#	perf = cbind(perf, as.factor(rep(sndpaths, each=10)))
#	colnames(perf) = c("perf", "path")
#	pdf(file=file.path(DIR, "F1SndHier.pdf"))
#	boxplot(perf~path,data=perf,names=gsub("pathway", "", sndpaths),las=2,ylim=c(0,1), ylab="F1", main=paste("pathways (2nd hier., n = ", length(sndpaths), ")"), las=3, cex.axis=0.5)
#	dev.off()		
	pdf(file=file.path(DIR, "PerformanceAllPathways.pdf"))
	idx = setdiff(grep("F1",rownames(stats)), grep("F1\\.sample",rownames(stats)))
	perf = 	as.vector((stats[idx,]))
	perf = cbind(perf, as.factor(rep(colnames(alldat$labels), each=10)))
	colnames(perf) = c("perf", "path")
	boxplot(perf~path,data=perf,names=colnames(alldat$labels),ylim=c(0,1),ylab="F1",main=paste("all pathways (n = ", length(colnames(alldat$labels)), ")"), las=3, cex.axis=0.5)
	#
	idx = setdiff(grep("prec",rownames(stats)), grep("prec\\.sample",rownames(stats)))
	perf = 	as.vector((stats[idx,]))
	perf = cbind(perf, as.factor(rep(colnames(alldat$labels), each=10)))
	colnames(perf) = c("perf", "path")
	boxplot(perf~path,data=perf,names=colnames(alldat$labels),ylim=c(0,1),ylab="precision",main=paste("all pathways (n = ", length(colnames(alldat$labels)), ")"), las=3, cex.axis=0.5)
	#
	idx = setdiff(grep("rec",rownames(stats)), grep("rec\\.sample",rownames(stats)))
	perf = 	as.vector((stats[idx,]))
	perf = cbind(perf, as.factor(rep(colnames(alldat$labels), each=10)))
	colnames(perf) = c("perf", "path")
	boxplot(perf~path,data=perf,names=colnames(alldat$labels),ylim=c(0,1),ylab="recall",main=paste("all pathways (n = ", length(colnames(alldat$labels)), ")"), las=3, cex.axis=0.5)
	dev.off()
	list(mean_loss=means["loss"], sd_loss=sds["loss"], precision=means[grep("preci",names(means))], sd_precision=sds[grep("preci",names(sds))], recall=means[grep("reca",names(means))], sd_recall=sds[grep("reca",names(sds))], F1=means[grep("F1",names(means))], sd_F1=sds[grep("F1",names(sds))])
}

run.crossvalidation = function(nfolds=10, repeats=10, stratified=TRUE, signaltrans.only=FALSE, minnmap=ifelse(signaltrans.only, 15, 30), nbag=11, level1Only="Metabolism", level2Only="Genetic Information Processing", organism="hsa", gene2Domains=NULL, seed=1234, mc.cores=8, DIR="."){
	set.seed=(seed)
	if(!signaltrans.only)
		alldat = gene2pathway:::buildTrainingSet(minnmap=minnmap, level1Only=level1Only, level2Only=level2Only, remove.duplicates=TRUE, organism=organism, gene2Domains=gene2Domains)
	else
		alldat = gene2pathway:::buildTrainingSet.signaltrans(minnmap=minnmap, remove.duplicates=TRUE, organism=organism, gene2Domains=gene2Domains)			
	features = alldat$features
	labels = alldat$labels			
	n = nrow(labels)	
	cv = matrix(0,nrow=n*repeats, ncol=ncol(labels))
	groups = list()
	used_domains = list()	
	kk = 1
	for(r in 1:repeats){
		perm = sample(1:n)
		for(k in 1:nfolds){
			cat("CV fold ", k,"\n\n")
			tst = perm[seq(k, n, by=nfolds)]
			trn = setdiff(1:n, tst)			
			tmp = alldat
			tmp$features = alldat$features[trn,]
			tmp$labels = alldat$labels[trn,]
			train = gene2pathway:::trainallBag(tmp, nbag=nbag, stratified=stratified, mc.cores=mc.cores)
			test = (gene2pathway:::predict.gene2pathway(train, features[tst,], mc.cores=mc.cores)$votes.matrix > 0.5)*1
			err = sapply(1:nrow(test), function(t) gene2pathway:::loss(labels[tst[t],], test[t,], alldat$treesizes, alldat$parentPaths))
			cat("\ntest error = ", median(err)," (min = ", min(err), "max = ", max(err), "avg = ", mean(err), ")\n")						
			used_domains[[kk]] = train$domains
			cv[(r - 1)*n + tst,] = test
			groups[[kk]] = tst
			kk = kk + 1
		}				
	}	
	eval = gene2pathway:::evaluate.cv(alldat, cv, groups, signaltrans.only=signaltrans.only, nfolds=nfolds, DIR=DIR)	
	list(cv=cv, groups=groups, used_domains=used_domains, evaluation=eval)
}

==> gene2pathway_2.12.0.tar.gz: gene2pathway/R/run.crossvalidation.R <==
# TODO: Add comment
# 
# Author: frohlich
###############################################################################
evaluate.cv = function(alldat, cvpred, groups, signaltrans.only=FALSE, nfolds=10, DIR="."){
	labels = alldat$labels
	n = nrow(labels)	
	stats.raw = sapply(1:length(groups), function(grno){
				g = groups[[grno]]
				r = (grno-1) %/% nfolds
				myidx = r*n + g
				tp = sapply(1:ncol(labels), function(l) sum((labels[g,l] == 1) & (cvpred[myidx,l] == 1))*1)
				fp = sapply(1:ncol(labels), function(l) sum((labels[g,l] == 0) & (cvpred[myidx,l] == 1))*1)
				fn = sapply(1:ncol(labels), function(l) sum((labels[g,l] == 1) & (cvpred[myidx,l] == 0))*1)
				prec = tp / (tp + fp)
				prec[is.na(prec)] = 1
				rec = tp / (tp + fn)
				rec[is.na(rec)] = 1
				F1 = 2*prec*rec / (prec + rec)
				F1[is.na(F1)] = 0			
				
				lo = sapply(1:length(g), function(i) loss(labels[g[i],], cvpred[myidx[i],], alldat$treesizes, alldat$parentPaths))
				tp2 = sum(tp)
				fp2 = sum(fp)
				fn2 = sum(fn)
				prec.sample = tp2 / (tp2 + fp2)
				prec.sample[is.na(prec.sample)] = 1
				rec.sample = tp2 / (tp2 + fn2)
				rec.sample[is.na(rec.sample)] = 1
				F1.sample = 2*prec.sample*rec.sample / (prec.sample + rec.sample)
				F1.sample[is.na(F1.sample)] = 0		
				lss = median(lo)
				names(prec) = colnames(labels)
				names(rec) = colnames(labels)
				names(F1) = colnames(labels)						
				list(preci=prec, reca=rec, F1=F1, loss=lss, F1.sample=F1.sample, prec.sample=prec.sample, rec.sample=rec.sample)
			})				
	stats.raw = apply(stats.raw,2,unlist)
	repeats = ncol(stats.raw) / nfolds
	stats = unlist(sapply(seq(1, ncol(stats.raw), by=nfolds), function(s1) rowMeans(stats.raw[,s1:(s1+(nfolds-1))])))			
	means = rowMeans(stats)
	sds = apply(stats,1,sd,na.rm=TRUE)
		
	pdf(file=file.path(DIR, "PerformanceSummary.pdf"))
	boxplot(1-stats["loss",],stats[grep("prec\\.sample",rownames(stats)),], stats[grep("rec\\.sample",rownames(stats)),], stats[grep("F1\\.sample",rownames(stats)),], ylim=c(0,1), names=c("accuracy","precision","recall","F1"), main=paste("pathways (n = ", ncol(labels), ")"))
	dev.off()
	
	rootpaths = colnames(labels)[alldat$kegg_hierarchy$parentsLev1]
	idx = rownames(stats)[unlist(sapply(rootpaths, function(r) grep(r, rownames(stats))))]
	if(signaltrans.only)
		idx = setdiff(idx, rownames(stats)[unlist(sapply(rootpaths, function(r) grep(paste(r,"\\.[0-9]*",sep=""), rownames(stats))))])
	pdf(file=paste("PerformanceSummaryallTopHier.pdf",sep=""))
	boxplot(as.vector(stats[idx[grep("preci",idx)],]), as.vector(stats[idx[grep("reca",idx)],]), as.vector(stats[idx[grep("F1",idx)],]), ylim=c(0,1), names=c("precision","recall","F1"), main=paste("all pathways (top hierarchy, n = ", length(rootpaths), ")"))
	dev.off()		
#	perf = 	as.vector((stats[idx[grep("F1",idx)],]))
#	perf = cbind(perf, as.factor(rep(rootpaths, each=10)))
#	colnames(perf) = c("perf", "path")
#	pdf(file=file.path(DIR, "F1TopHier.pdf"))
#	boxplot(perf~path,data=perf,names=gsub("pathway", "", rootpaths),ylim=c(0,1),ylab="F1",main=paste("all pathways (top hier., n = ", length(rootpaths), ")"), las=3, cex.axis=0.5)
#	dev.off()
	
	sndpaths = colnames(labels)[alldat$kegg_hierarchy$parentsLev2]
	idx = setdiff(rownames(stats)[sapply(sndpaths, function(r) grep(r, rownames(stats)))], idx)
	pdf(file=file.path(DIR, "PerformanceSummaryall2ndHier.pdf"))
	boxplot(as.vector(stats[idx[grep("preci",idx)],]), as.vector(stats[idx[grep("reca",idx)],]), as.vector(stats[idx[grep("F1",idx)],]), ylim=c(0,1), names=c("precision","recall","F1"), main=paste("pathways (2nd hier., n = ", length(sndpaths), ")"))
	dev.off()
	
#	perf = 	as.vector((stats[idx[grep("F1",idx)],]))
#	perf = cbind(perf, as.factor(rep(sndpaths, each=10)))
#	colnames(perf) = c("perf", "path")
#	pdf(file=file.path(DIR, "F1SndHier.pdf"))
#	boxplot(perf~path,data=perf,names=gsub("pathway", "", sndpaths),las=2,ylim=c(0,1), ylab="F1", main=paste("pathways (2nd hier., n = ", length(sndpaths), ")"), las=3, cex.axis=0.5)
#	dev.off()		
	pdf(file=file.path(DIR, "PerformanceAllPathways.pdf"))
	idx = setdiff(grep("F1",rownames(stats)), grep("F1\\.sample",rownames(stats)))
	perf = 	as.vector((stats[idx,]))
	perf = cbind(perf, as.factor(rep(colnames(alldat$labels), each=10)))
	colnames(perf) = c("perf", "path")
	boxplot(perf~path,data=perf,names=colnames(alldat$labels),ylim=c(0,1),ylab="F1",main=paste("all pathways (n = ", length(colnames(alldat$labels)), ")"), las=3, cex.axis=0.5)
	#
	idx = setdiff(grep("prec",rownames(stats)), grep("prec\\.sample",rownames(stats)))
	perf = 	as.vector((stats[idx,]))
	perf = cbind(perf, as.factor(rep(colnames(alldat$labels), each=10)))
	colnames(perf) = c("perf", "path")
	boxplot(perf~path,data=perf,names=colnames(alldat$labels),ylim=c(0,1),ylab="precision",main=paste("all pathways (n = ", length(colnames(alldat$labels)), ")"), las=3, cex.axis=0.5)
	#
	idx = setdiff(grep("rec",rownames(stats)), grep("rec\\.sample",rownames(stats)))
	perf = 	as.vector((stats[idx,]))
	perf = cbind(perf, as.factor(rep(colnames(alldat$labels), each=10)))
	colnames(perf) = c("perf", "path")
	boxplot(perf~path,data=perf,names=colnames(alldat$labels),ylim=c(0,1),ylab="recall",main=paste("all pathways (n = ", length(colnames(alldat$labels)), ")"), las=3, cex.axis=0.5)
	dev.off()
	list(mean_loss=means["loss"], sd_loss=sds["loss"], precision=means[grep("preci",names(means))], sd_precision=sds[grep("preci",names(sds))], recall=means[grep("reca",names(means))], sd_recall=sds[grep("reca",names(sds))], F1=means[grep("F1",names(means))], sd_F1=sds[grep("F1",names(sds))])
}

run.crossvalidation = function(nfolds=10, repeats=10, stratified=TRUE, signaltrans.only=FALSE, minnmap=ifelse(signaltrans.only, 15, 30), nbag=11, level1Only="Metabolism", level2Only="Genetic Information Processing", organism="hsa", gene2Domains=NULL, branch="Signal Transduction", seed=1234, mc.cores=8, DIR="."){
	set.seed=(seed)
	if(!signaltrans.only)
		alldat = gene2pathway:::buildTrainingSet(minnmap=minnmap, level1Only=level1Only, level2Only=level2Only, remove.duplicates=TRUE, organism=organism, gene2Domains=gene2Domains, branch=branch)
	else
		alldat = gene2pathway:::buildTrainingSet.signaltrans(minnmap=minnmap, remove.duplicates=TRUE, organism=organism, gene2Domains=gene2Domains)			
	features = alldat$features
	labels = alldat$labels			
	n = nrow(labels)	
	cv = matrix(0,nrow=n*repeats, ncol=ncol(labels))
	groups = list()
	used_domains = list()	
	kk = 1
	for(r in 1:repeats){
		perm = sample(1:n)
		for(k in 1:nfolds){
			cat("CV fold ", k,"\n\n")
			tst = perm[seq(k, n, by=nfolds)]
			trn = setdiff(1:n, tst)			
			tmp = alldat
			tmp$features = alldat$features[trn,]
			tmp$labels = alldat$labels[trn,]
			train = gene2pathway:::trainallBag(tmp, nbag=nbag, stratified=stratified, mc.cores=mc.cores)
			test = (gene2pathway:::predict.gene2pathway(train, features[tst,], mc.cores=mc.cores)$votes.matrix > 0.5)*1
			err = sapply(1:nrow(test), function(t) gene2pathway:::loss(labels[tst[t],], test[t,], alldat$treesizes, alldat$parentPaths))
			cat("\ntest error = ", median(err)," (min = ", min(err), "max = ", max(err), "avg = ", mean(err), ")\n")						
			used_domains[[kk]] = train$domains
			cv[(r - 1)*n + tst,] = test
			groups[[kk]] = tst
			kk = kk + 1
		}				
	}	
	eval = gene2pathway:::evaluate.cv(alldat, cv, groups, signaltrans.only=signaltrans.only, nfolds=nfolds, DIR=DIR)	
	list(cv=cv, groups=groups, used_domains=used_domains, evaluation=eval)
}
